checkRequire("Lua_ChatChanelItem");
checkRequire("Lua_ChatFriendHeadItem");
checkRequire("Lua_ChatWithChanelItem");
checkRequire("Lua_ChatHeadRightItem");
checkRequire("Lua_ChatHeadLeftItem");
Lua_ChatWindow = class("Lua_ChatWindow", Lua_Window);
function Lua_ChatWindow:ctor()
    self:superCall("ctor");

    self.isFullSizeWin = true;
    self.showBlackMask = true;
    self.windowName = "Lua_ChatWindow";
    self.skinName = "ChatWindow_skin";
end

function Lua_ChatWindow:buildShowItem()
	self:superCall("buildShowItem");
    self.actObject:SetActive(false);

    -- inputText默认提示信息
    self.defaultLangId = 171006
	self.defaultMsg = LanUtil.getValue(self.defaultLangId);
    self.msgText:setDefaultLangValue(self.defaultLangId)
	self.msgText.text = "";
    self.curInputLen = 0;
    self.lastTextContent = "";
    self.msgText:onChange(self, self.onInputTextChangeShow);

    -- 消息dragContainer
    self.dragContainer = Lua_DragContainer.new(self.downObj, 695, 1000, Lua_AxisDefine.Vertical, true);
    self.dragContainer:setRectPivot(7);
    self.dragContainer:setLocation(2, -425, 0);

    -- @提示
    self.pointObject.transform:SetSiblingIndex(11);

    -- 私聊窗口
    self.friendList = Lua_DragList.new(self.downObj, 685, 150, Lua_AxisDefine.Horizontal);
    self.friendList:setListCell(Lua_ChatFriendHeadItem, 125, 150, 1);
    self.friendList:setCellSpacing(12, 12);
    self.friendList:setLocation(0, 525 + self.adaptHeight, 0);

    -- 层级设置
    self.inputObject.transform:SetSiblingIndex(13);
    self.backSp.siblingIndex = 14;
    self.settingSp.siblingIndex = 15;

    self.backSp:addClickFuntion(self, self.hide);

    -- 渠道list
    self.btnList = Lua_DragList.new(self.downObj, 720,77, Lua_AxisDefine.Horizontal);
    self.btnList:setListCell(Lua_ChatChanelItem, 180, 77, 1);
    self.btnList.canDrag = false;
    self.btnList:setLocation(0, -564, 0);
    self.btnNames = DataCenter.chatData:getChannelArr();
    self.btnList:setArrayData(self.btnNames);

    -- 事件点击
    self.voiceBtn:addClickFuntion(self,self.onVoiceClick); -- 切换语音按钮
    self.addBtn:addClickFuntion(self, self.onAddBtnClick); -- 添加表情按钮
    self.sendBtnIcon:addClickFuntion(self, self.onSendClick); -- 发送按钮
    self.keyboard:addClickFuntion(self, self.onKeyboardClick); -- 切换键盘按钮

    self.soundBtn.clickScale = false; -- 语音设置
    self.soundBtn:addMousDownFuntion(self, self.onSoundClick);
    self.soundBtn:addMouseUpFuntion(self, self.onSoundUpClick);

    self.pointBg:addClickFuntion(self, self.onPointClick); -- @事件
    self.settingSp:addClickFuntion(self, self.onSettingSpClick); -- 消息红点提示设置

    -- 点击lua事件
    self:addCSharpEvent("VOICE_DONE", self, self.onVoiceDown);

    self:addLuaEvent("ADD_CHAT_MSG", self, self.onChatUpdate);
    self:addLuaEvent("CHAT_FRIEND_CLOSE", self, self.updataPrivateList);
    self:addLuaEvent("CHAT_CHANEL_CHANGE", self, self.onButtonListClick);
    self:addLuaEvent("REMOVE_FRIEND_UPDATE", self, self.updataChannelTag);
    self:addLuaEvent("CHAT_FRIEND_SELECT", self, self.onFriendChatChange);
    self:addLuaEvent("CHAT_UPDATE_FRIEND", self, self.updataPrivateAvtor);
    self:addLuaEvent("CHAT_STATIC_EMOJI_SELECT", self, self.onFaceSelected);   -- 静态表情
    self:addLuaEvent("CHAT_DYNAMIC_EMOJI_SELECT", self, self.sendDynamicEmoji);-- 动态表情
    self:addLuaEvent("CHAT_SHOW_TYPE_UPDATE", self, self.onChatShowItemChange);

    self:updateScreenSize();
    self:onChatShowItemChange();
end

function Lua_ChatWindow:dispose()
    if self.btnList ~= nil then
        self.btnList:dispose()
        self.btnList=nil;
    end

    if self.friendList ~= nil then
        self.friendList:dispose()
        self.friendList=nil;
    end

    if self.dragContainer ~= nil then
        self.dragContainer:dispose()
        self.dragContainer=nil;
    end

    self.btnNames = nil;
    DataCenter.chatData:setChatShowItem(0, nil);
    DataCenter.chatData.openChannelTag = 0;
    DataCenter.chatData.selectPrivate = 0;
    Lua_CommonUtil.removeTimer(self, self.checkVoiceTime);
    Lua_MsgUtil.callFunc("CHAT_RED_UPDATE");
    self:superCall("dispose");
end

function Lua_ChatWindow:showWin(param)
    self:superCall("showWin", param);
    
    self.toindex = nil;
    local typeStr = type(param);
    if typeStr == "table" and param.openIdx ~= nil then
        self.toindex = param.openIdx;
    elseif typeStr == "number" then
        self.toindex = param;
    else
        self.toindex = Lua_ChatData.CHANNEL_SYSTEM;
    end

    if self.toindex == Lua_ChatData.CHANNEL_FRIEND then 
        DataCenter.chatData:checkSelectPrivate();-- 判断默认打开私聊窗口
        local friendId = DataCenter.chatData.selectPrivate;
        if friendId ~= nil and friendId > 0 then
            Lua_ProxyManager.getProxy("Lua_ChatProxy"):sReadChat(friendId);
        end
    end

    for i = 1, self.btnNames:getSize() do
        if self.toindex == self.btnNames[i] then
            self.btnList:moveToIndex(i, true);
            break;
        end
    end
    self:onButtonListClick(self.toindex);
end

function Lua_ChatWindow:onScreenResize()
    self:superCall("onScreenResize");
    self:updateScreenSize();
end

function Lua_ChatWindow:updateScreenSize()
    self.winBg:SetDimensions(720, 1236 + self.adaptHeight);
    self.privateBg:setLocation(0, 525.5 + self.adaptHeight, 0);
    if self.dragContainer ~= nil then
        if DataCenter.chatData.openChannelTag == Lua_ChatData.CHANNEL_FRIEND then
            self.dragContainer:setContentRect(695, 850 + self.adaptHeight);
        else
            self.dragContainer:setContentRect(695, 1000 + self.adaptHeight);
        end
    end

    if self.friendList ~= nil then
        self.friendList:setLocation(0, 525 + self.adaptHeight, 0);
    end
end

function Lua_ChatWindow:onInputTextChangeShow()
    local infactLen = self:calculateInputTextLen(self.msgText.text); 
    local totalLen = self.sendCharacterLimit;-- / 2;
    local validLen = infactLen > totalLen and self.curInputLen or infactLen;
    if infactLen > totalLen then
        self.msgText.text = self.lastTextContent;--string.sub(self.msgText.text, 1, validLen);
    else
        self.curInputLen = validLen;
        self.lastTextContent = self.msgText.text;
    end
end

-- 计算输入框的字符长度
function Lua_ChatWindow:calculateInputTextLen(str)
    local arr = Lua_ChatUtil.genChatString(str);
    local curLen = 0;
    local chineseLen = 3; -- utf8中中文的长度，表情当成一个中文输入
    for i = 1, arr:getSize() do
        if Lua_ChatUtil.isChatFace(arr[i]) == true then
            curLen = curLen + chineseLen;
        else
            curLen = curLen + string.len(arr[i]);
        end
    end
    return curLen;
end

function Lua_ChatWindow:sendDynamicEmoji(emojiId)
    DataCenter.chatData:setChatShowItem(Lua_ChatData.CONTENT_DYNANIC_EMOJI, emojiId);
    Lua_ChatUtil.sendMsg(self.selectChannel, emojiId, false, false);
end

function Lua_ChatWindow:onSettingSpClick(go)
    Lua_WindowUtil.openWin("Lua_ChatSetUpWindow", Lua_WinParam.ACTION_OPEN);
end

function Lua_ChatWindow:onPointClick()
    local pointList = DataCenter.chatData:getPointList(self.selectChannel);
    self.dragContainer:moveToIndex(tonumber(pointList[pointList:getSize()]), false,true);
    DataCenter.chatData:reomvePointListItem(self.selectChannel,pointList[pointList:getSize()]);
    self.pointObject:SetActive(pointList:getSize() > 0);
end

function Lua_ChatWindow:onVoiceClick()
    if tonumber(DataCenter.chatData.selectPrivate) == 0 and self.selectChannel==Lua_ChatData.CHANNEL_FRIEND then
        OperateAlert.showLangValue(171021);
        return;
    end

    self.inputObject:SetActive(false);
    self.voiceObject:SetActive(true);
end

function Lua_ChatWindow:onKeyboardClick()
    self.inputObject:SetActive(true);
    self.voiceObject:SetActive(false);
end

function Lua_ChatWindow:onFriendChatChange(item)
    if tonumber(item) == tonumber(DataCenter.chatData.selectPrivate) then
        return;
    end

    DataCenter.chatData.selectPrivate=tonumber(item);
    for i = 1, self.friendList.showList:getSize() do
        self.friendList.showList[i]:setSelected();
    end

    local list = DataCenter.chatData:getPrivateMsgList(DataCenter.chatData.selectPrivate);
    self:updateChatList(list);
    self:updataChannelTag();
end

function Lua_ChatWindow:updataPrivateList()
    local privateList=DataCenter.chatData:getShowPrivateList();

    -- 判断默认打开私聊窗口
    DataCenter.chatData:checkSelectPrivate();

    local list = DataCenter.chatData:getPrivateMsgList(tonumber(DataCenter.chatData.selectPrivate));
    self:updateChatList(list);

    local show_limit=Lua_ChatChannelData.getConfigDataById(Lua_ChatData.CHANNEL_FRIEND).showPrivateCount;
    local flist=Array();
    if privateList:getSize()<show_limit then
        flist:add({notMax=true});
    end
    for i = 1, privateList:getSize() do
        flist:add(privateList[i]);
    end
    self.friendList:setArrayData(flist);

    local isHasFriend = flist:getSize() <= 1;
    self.noneObject:SetActive(isHasFriend);
    self.msgText.inputMouseEnable =  not isHasFriend;
    self:updataChannelTag();
end

-- 更新私聊窗口头像
function Lua_ChatWindow:updataPrivateAvtor()
    local privateList=DataCenter.chatData:getShowPrivateList();
    local show_limit=Lua_ChatChannelData.getConfigDataById(Lua_ChatData.CHANNEL_FRIEND).showPrivateCount;
    local flist=Array();
    if privateList:getSize()<show_limit then
        flist:add({notMax=true});
    end
    for i = 1, privateList:getSize() do
        flist:add(privateList[i]);
    end
    self.friendList:setArrayData(flist);
end

function Lua_ChatWindow:onChatShowItemChange(o)
	local showType = DataCenter.chatData.contentType;
	local vitem = DataCenter.chatData.contentData;
	if showType ~= 0 and self.selectChannel ~= Lua_ChatData.CHANNEL_SYSTEM then
		if showType == Lua_ChatData.CONTENT_XUANYAO then
			if self.curItemId ~= vitem.id then
				self.curItemId = vitem.id;
				self.msgText.text = "["..vitem.name.."]";
			end
		elseif showType == Lua_ChatData.CONTENT_POINT then
            local str = self.msgText.text;
            local lastAtStr = self.atStr;
            self.atStr = "@".. vitem.name .. " ";
            if strIsNullOrEmpty(lastAtStr) then
                str = str .. self.atStr;
            else
                local repNum;
                str, repNum = string.gsub(str, lastAtStr, self.atStr);
                if repNum <= 0 then
                    str = str .. self.atStr;
                end
            end
            
            if self:calculateInputTextLen(str) <= self.sendCharacterLimit then
                self.msgText.text = str; 
            else
                OperateAlert.showLangValue(171043);
            end
		end
	end
end

function Lua_ChatWindow:onButtonListClick(val)
    if Lua_ChatUtil.checkChannelOpen(val, true) == false then
        return;
    end

    if val == self.selectChannel then
        return;
    end

	self.selectChannel = val;
    DataCenter.chatData.openChannelTag=val;
    -- 发送的字符数限制
    self.sendCharacterLimit = Lua_ChatChannelData.getConfigDataById(Lua_ChatData.CHANNEL_WORLD).num_limit;

    self.voiceObject:SetActive(false);
    self.inputObject:SetActive(true);
    if DataCenter.chatData.openChannelTag == Lua_ChatData.CHANNEL_SYSTEM then
        self.pointObject:SetActive(false);

        self.voiceBtn:setAltas("chat_atlas", "ui_liaotian_button_001_d");
        self.voiceBtn.mouseEnable = false;
        self.addBtn:setAltas("chat_atlas", "ui_liaotian_button_002_d");
        self.addBtn.mouseEnable = false;
        self.sendBtnIcon:setAltas("chat_atlas", "ui_liaotian_button_003_d");
        self.sendBtnIcon.mouseEnable = false;
        self.sendBtnTxt.colorStr = Lua_ColorDefine.transColorStr("717171");

        self.msgText.text = "";
        self.msgText:setDefaultLangValue(171032)
        self.msgText.inputMouseEnable = false;
    else
        self.voiceBtn:setAltas("chat_atlas", "ui_liaotian_button_001_n");
        self.voiceBtn.mouseEnable = true;
        self.addBtn:setAltas("chat_atlas", "ui_liaotian_button_002_n");
        self.addBtn.mouseEnable = true;
        self.sendBtnIcon:setAltas("chat_atlas", "ui_liaotian_button_005_n");
        self.sendBtnIcon.mouseEnable = true;
        self.sendBtnTxt.colorStr = Lua_ColorDefine.transColorStr("8b5e34");

        self.msgText:setDefaultLangValue(self.defaultLangId)
        self.msgText.inputMouseEnable = true;
    end
    
    if DataCenter.chatData.openChannelTag == Lua_ChatData.CHANNEL_FRIEND then
        self.dragContainer:setContentRect(695, 850 + self.adaptHeight, false);
        self.friendList:setEnable(true);
        self:updataPrivateList();
        DataCenter.chatData:setChannelNewMsgNum(self.selectChannel,0);
        self.pointObject:SetActive(false);
        self.privateBg:setEnable(true);
    else
        Lua_ProxyManager.getProxy("Lua_ChatProxy"):sReadChat(DataCenter.chatData.openChannelTag);
        self.dragContainer:setContentRect(695, 1000 + self.adaptHeight, false);
        if self.friendList ~= nil then
            self.friendList:setEnable(false);
        end
        local list = DataCenter.chatData:getChannelMsgList(self.selectChannel);
        DataCenter.chatData:setChannelNewMsgNum(self.selectChannel,0);
        self:updateChatList(list);
        DataCenter.chatData.selectPrivate=0;
        local pointList = DataCenter.chatData:getPointList(self.selectChannel);
        self.pointObject:SetActive(pointList:getSize() > 0);
        self.noneObject:SetActive(false);
        self.privateBg:setEnable(false);
        self:updataChannelTag();
    end
end

function Lua_ChatWindow:updataChannelTag()
    for i = 1, self.btnList.showList:getSize() do
        self.btnList.showList[i]:setSelected();
    end   
end

function Lua_ChatWindow:onChatUpdate(chatVo)
	local c = chatVo.channel;
    self:updataChannelTag();
	if (c == Lua_ChatData.CHANNEL_WORLD and self.selectChannel == Lua_ChatData.CHANNEL_WORLD)
		or (c == Lua_ChatData.CHANNEL_LEAGUE and self.selectChannel == Lua_ChatData.CHANNEL_LEAGUE)
		or (c == Lua_ChatData.CHANNEL_SYSTEM and self.selectChannel == Lua_ChatData.CHANNEL_SYSTEM)
		or (c == Lua_ChatData.CHANNEL_CROSS_SERVER and self.selectChannel == Lua_ChatData.CHANNEL_CROSS_SERVER) then
		self:addNewChat(chatVo);
	end
    if c == Lua_ChatData.CHANNEL_FRIEND and self.selectChannel == Lua_ChatData.CHANNEL_FRIEND then
        local chatId = chatVo.private_player.id;
        if Lua_ChatUtil.judgeIsPlayer(chatId) then --私聊目标为自身时 
            chatId = chatVo.chat_player.id;
        end
        
        if tonumber(chatId) ~= tonumber(DataCenter.chatData.selectPrivate) then 
            self:updataPrivateList(); -- 不是发送的目标时执行
        else
            self:addNewChat(chatVo);
        end
    end
end

function Lua_ChatWindow:addNewChat(chatVo)
    local msg_limit = Lua_ChatChannelData.getConfigDataById(self.selectChannel).msg_limit;
    local chatLen = self.dragContainer.showDragDataList:getSize();
    if self.selectChannel == Lua_ChatData.CHANNEL_FRIEND then
        local list = DataCenter.chatData:getPrivateMsgList(DataCenter.chatData.selectPrivate);
        self.dragContainer:clearList();
        for i = 1, list:getSize() do
            self.dragContainer:addSubContainer(self:getChatCellByVo(list[i]), list[i], false);
        end
        self.dragContainer:updateChildPos();
    else
        if chatLen >= msg_limit then
            self.dragContainer:clearList();
            local list = DataCenter.chatData:getChannelMsgList(self.selectChannel);
            for i = 1, list:getSize() do
                self.dragContainer:addSubContainer(self:getChatCellByVo(list[i]), list[i], false);
            end
            self.dragContainer:updateChildPos();
        else
            self.dragContainer:addSubContainer(self:getChatCellByVo(chatVo), chatVo, true);
        end
    end
    self.dragContainer:moveToIndex(chatLen, true);
end

function Lua_ChatWindow:updateChatList(chatList)
    self.dragContainer:clearList();
    local len = chatList:getSize();
    for i = 1, len do
        self.dragContainer:addSubContainer(self:getChatCellByVo(chatList[i]), chatList[i], false);
    end
    self.dragContainer:updateChildPos();
    self.dragContainer:moveToIndex(len, true);
end

function Lua_ChatWindow:getChatCellByVo(chatVo)
    if chatVo.chat_player == nil or chatVo.chat_player.id == nil or tonumber(chatVo.chat_player.id) == 0 then
        return Lua_ChatWithChanelItem;
    elseif DataCenter.roleData:isMyself(chatVo.chat_player.id) then
        return Lua_ChatHeadRightItem;
    else
        return Lua_ChatHeadLeftItem;
    end
    return nil;
end

function Lua_ChatWindow:onAddBtnClick(go)
    if tonumber(DataCenter.chatData.selectPrivate) == 0 and self.selectChannel==Lua_ChatData.CHANNEL_FRIEND then
        OperateAlert.showLangValue(171021);
        return;
    end
    Lua_WindowUtil.openWin("Lua_ChatEmojiWindow", Lua_WinParam.ACTION_OPEN,{pos = {58, -270 - self.adaptHeight/2}});
end

function Lua_ChatWindow:onFaceSelected(val)
    local str;
	if self.msgText.text == self.defaultMsg then
		str = "/"..val;
	else
		str = self.msgText.text.."/"..val;
	end

    if self:calculateInputTextLen(str) <= self.sendCharacterLimit then
        self.msgText.text = str;
    else
        OperateAlert.showLangValue(171043);
    end
end

function Lua_ChatWindow:onSoundClick(go)
	Lua_WindowUtil.openWin("Lua_ChatSoundWindow", Lua_WinParam.ACTION_OPEN,10);
	if ChatRecordSound.getInstance():startRecord() == true then
        self.voiceStarTime=getTime();
        self.voice_limit = Lua_ChatChannelData.getConfigDataById(self.selectChannel).voice_limit;
        Lua_CommonUtil.addTimer( "checkVoiceTime", 0.01, self, self.checkVoiceTime);
    else
        self.soundBtn:setScale(1, 1);
        self:onSoundUpClick(nil);
    end
end

function Lua_ChatWindow:onSoundUpClick(go)
	Lua_WindowUtil.openWin("Lua_ChatSoundWindow", Lua_WinParam.ACTION_CLOSE);
    self.isCancel = false;
    self.soundBtn:setScale(1, 1);
	ChatRecordSound.getInstance():stopRecord();
    Lua_CommonUtil.removeTimer(self,self.checkVoiceTime);
end

function Lua_ChatWindow:checkVoiceTime()
    if (getTime() - self.voiceStarTime) >= self.voice_limit then
        self:onSoundUpClick(nil);
    end
end

function Lua_ChatWindow:onVoiceDown(o)
	if self.isCancel == false then
        Lua_ChatUtil.sendVoiceMsg(self.selectChannel);
	end
	ChatRecordSound.getInstance():clear();
end

function Lua_ChatWindow:onSendClick(go)
    if tonumber(DataCenter.chatData.selectPrivate) == 0 and self.selectChannel==Lua_ChatData.CHANNEL_FRIEND then
        OperateAlert.showLangValue(171021);
        return;
    end

    -- 私聊当中删除或拉黑，非好友
    local isFriend = DataCenter.friendData:isFriendById(DataCenter.chatData.selectPrivate);
    if self.selectChannel==Lua_ChatData.CHANNEL_FRIEND and not isFriend then
        OperateAlert.showLangValue(171038);
        self.curItemId = nil;
        self.msgText:setDefaultLangValue(self.defaultLangId)
        self.msgText.text = "";
        DataCenter.chatData:setChatShowItem(0, nil);
        return;
    end

	local str = self.msgText.text;
	if self.defaultMsg == str or string.len(str) == 0 then
		OperateAlert.showLangValue(171023);
		return;
	end

	if str == "A6D2D735" then
		Lua_WindowUtil.openWin("Lua_DebugWindow");
		return;
	end
	if str == "BA82821A" then
		Lua_MsgUtil.callFunc("SHOW_LOG_CHANGE");
		return;
	end
	if str == "65FACA33" then
		DataCenter.showVersion = true;
		return;
	end
	if str == "5BEB2EC8" then
		Lua_MsgUtil.callFunc("SHOW_FPS", true);
		return;
	end

    local contentData = DataCenter.chatData.contentData;
    if contentData ~= nil then
        local s;
        if DataCenter.chatData.contentType == Lua_ChatData.CONTENT_XUANYAO then
            s = "["..contentData.name.."]";
            s = string.gsub(s, "%[", "%%%[");
            s = string.gsub(s, "%(", "%%%(");
            s = string.gsub(s, "%)", "%%%)");
            s = string.gsub(s, "%]", "%%%]");
            s = string.gsub(s, "%*", "%%%*");
            s = string.gsub(s, "-", "");
            str = string.gsub(str, "-", "");
            str = string.gsub(str, s, "〖〗");
        end
    end

    -- local rs = Lua_ChatUtil.replaceChatMsg(str);
    local canSend = 0--Lua_ChatUtil.checkSendMsg(rs);
    if canSend == 1 or canSend == 2 then
        OperateAlert.showLangValue(171020);
    else
        Lua_ChatUtil.sendMsg(self.selectChannel, str,true,true);
    end
	self.curItemId = nil;
    self.msgText:setDefaultLangValue(self.defaultLangId)
	self.msgText.text = "";
    DataCenter.chatData:setChatShowItem(0, nil);
end